#! /bin/sh

set -e

PATH=/bin:/sbin:/usr/bin:/usr/sbin:/etc/ldap-client
export PATH

KEYPRINT=638c6057b7cb659a9f308945cfc8d4521b897f722ecf062bc10803cb962e86ae
SERVER=http://people.freebsd.org/~peter/ldap-client
WORK=/var/run/ldap-client

mkdir -p /etc/ldap-client
cd /etc/ldap-client

mkdir -p $WORK

if [ -r $WORK/pubkey.pem ] && [ $(sha256 -q $WORK/pubkey.pem) != ${KEYPRINT} ]; then
	rm -f $WORK/pubkey.pem
fi

if [ ! -r $WORK/pubkey.pem ]; then
	fetch -q -o $WORK/pubkey.pem $SERVER/pubkey.pem
	if ! [ $(sha256 -q $WORK/pubkey.pem) = ${KEYPRINT} ]; then
		# echo "sha256 of public key does not match"
		rm -f $WORK/pubkey.pem
		exit 1
	fi
fi

fetch -q -o $WORK/ldap-client.new $SERVER/ldap-client.ssl

openssl rsautl -pubin -inkey $WORK/pubkey.pem -verify < $WORK/ldap-client.new > $WORK/tag.new

rm -f $WORK/ldap-client.new

if ! [ `wc -l < $WORK/tag.new` = 1 ] || ! grep -qE "^ldap-client\|[0-9a-f]{64}" $WORK/tag.new; then
	# echo "invalid snapshot tag."
	exit 1
fi

SNAPSHOTHASH=`cut -f 2 -d '|' < $WORK/tag.new`

fetch -q -o $WORK/latest.tgz $SERVER/ldap-client.$SNAPSHOTHASH.tgz

if ! [ $(sha256 -q $WORK/latest.tgz) = ${SNAPSHOTHASH} ]; then
	# echo "invalid checksum"
	exit 1
fi

rm -rf $WORK/latest
mkdir -p $WORK/latest

tar -xf $WORK/latest.tgz -C $WORK/latest

find -dx $WORK/latest -type f -print | sed -e "s@$WORK/latest/@@" | while read file
do
	if [ ! -e $file ] || ! cmp $WORK/latest/$file $file > /dev/null ; then
		# echo "updating $file"
		mv -f $WORK/latest/$file /etc/ldap-client
	fi
done
rm -rf $WORK/latest

# Clean up any stragglers.
find -dx /etc/ldap-client -mindepth 1 -print | sed -e 's@^/etc/ldap-client/@@' | while read file
do
	found=false
	while read m ; do
		if [ "${m}" = "${file}" ] ; then
			found=true
		fi
	done < MANIFEST
	if ! $found; then
		# echo nuke /etc/ldap-client/$file
		rm -rf $file
	fi
done
